@@ -32,11 +32,25 @@ typedef struct
                            or d as used by Twisted Edwards curves.  */
   mpi_point_struct G;   /* Base point (generator).  */
   gcry_mpi_t n;         /* Order of G.  */
-  unsigned int h;       /* Cofactor.  */
   const char *name;     /* Name of the curve or NULL.  */
 } elliptic_curve_t;
 
 
+typedef struct
+{
+  elliptic_curve_t E;
+  mpi_point_struct Q; /* Q = [d]G  */
+} ECC_public_key;
+
+
+typedef struct
+{
+  elliptic_curve_t E;
+  mpi_point_struct Q;
+  gcry_mpi_t d;
+} ECC_secret_key;
+
+
 
 /* Set the value from S into D.  */
 static inline void
@@ -73,71 +87,55 @@ void _gcry_ecc_curve_free (elliptic_curve_t *E);
 elliptic_curve_t _gcry_ecc_curve_copy (elliptic_curve_t E);
 const char *_gcry_ecc_model2str (enum gcry_mpi_ec_models model);
 const char *_gcry_ecc_dialect2str (enum ecc_dialects dialect);
-unsigned char *_gcry_ecc_ec2os_buf (gcry_mpi_t x, gcry_mpi_t y, gcry_mpi_t p,
-                                    unsigned int *r_length);
 gcry_mpi_t   _gcry_ecc_ec2os (gcry_mpi_t x, gcry_mpi_t y, gcry_mpi_t p);
+gcry_err_code_t _gcry_ecc_os2ec (mpi_point_t result, gcry_mpi_t value);
 
-mpi_point_t  _gcry_ecc_compute_public (mpi_point_t Q, mpi_ec_t ec);
-gpg_err_code_t _gcry_ecc_mont_encodepoint (gcry_mpi_t x, unsigned int nbits,
-                                           int with_prefix,
-                                           unsigned char **r_buffer,
-                                           unsigned int *r_buflen);
-
+mpi_point_t  _gcry_ecc_compute_public (mpi_point_t Q, mpi_ec_t ec,
+                                       mpi_point_t G, gcry_mpi_t d);
 
 /*-- ecc.c --*/
 
 /*-- ecc-ecdsa.c --*/
-gpg_err_code_t _gcry_ecc_ecdsa_sign (gcry_mpi_t input, gcry_mpi_t k, mpi_ec_t ec,
+gpg_err_code_t _gcry_ecc_ecdsa_sign (gcry_mpi_t input, ECC_secret_key *skey,
                                      gcry_mpi_t r, gcry_mpi_t s,
                                      int flags, int hashalgo);
-gpg_err_code_t _gcry_ecc_ecdsa_verify (gcry_mpi_t input, mpi_ec_t ec,
-                                       gcry_mpi_t r, gcry_mpi_t s,
-                                       int flags, int hashalgo);
+gpg_err_code_t _gcry_ecc_ecdsa_verify (gcry_mpi_t input, ECC_public_key *pkey,
+                                       gcry_mpi_t r, gcry_mpi_t s);
 
 /*-- ecc-eddsa.c --*/
 gpg_err_code_t _gcry_ecc_eddsa_recover_x (gcry_mpi_t x, gcry_mpi_t y, int sign,
                                           mpi_ec_t ec);
 gpg_err_code_t _gcry_ecc_eddsa_encodepoint (mpi_point_t point, mpi_ec_t ctx,
                                             gcry_mpi_t x, gcry_mpi_t y,
-                                            int with_prefix,
                                             unsigned char **r_buffer,
                                             unsigned int *r_buflen);
 gpg_err_code_t _gcry_ecc_eddsa_ensure_compact (gcry_mpi_t value,
                                                unsigned int nbits);
-
-
+gpg_err_code_t _gcry_ecc_eddsa_decodepoint (gcry_mpi_t pk, mpi_ec_t ctx,
+                                            mpi_point_t result,
+                                            unsigned char **r_encpk,
+                                            unsigned int *r_encpklen);
 gpg_err_code_t _gcry_ecc_eddsa_compute_h_d (unsigned char **r_digest,
-                                            mpi_ec_t ec);
+                                            gcry_mpi_t d, mpi_ec_t ec);
 
-gpg_err_code_t _gcry_ecc_eddsa_genkey (mpi_ec_t ec, int flags);
+gpg_err_code_t _gcry_ecc_eddsa_genkey (ECC_secret_key *sk,
+                                       elliptic_curve_t *E,
+                                       mpi_ec_t ctx,
+                                       gcry_random_level_t random_level);
 gpg_err_code_t _gcry_ecc_eddsa_sign (gcry_mpi_t input,
-                                     mpi_ec_t ec,
+                                     ECC_secret_key *sk,
                                      gcry_mpi_t r_r, gcry_mpi_t s,
-                                     struct pk_encoding_ctx *ctx);
+                                     int hashalgo, gcry_mpi_t pk);
 gpg_err_code_t _gcry_ecc_eddsa_verify (gcry_mpi_t input,
-                                       mpi_ec_t ec,
+                                       ECC_public_key *pk,
                                        gcry_mpi_t r, gcry_mpi_t s,
-                                       struct pk_encoding_ctx *ctx);
-void reverse_buffer (unsigned char *buffer, unsigned int length);
-
+                                       int hashalgo, gcry_mpi_t pkmpi);
 
 /*-- ecc-gost.c --*/
-gpg_err_code_t _gcry_ecc_gost_sign (gcry_mpi_t input, mpi_ec_t ec,
+gpg_err_code_t _gcry_ecc_gost_sign (gcry_mpi_t input, ECC_secret_key *skey,
                                     gcry_mpi_t r, gcry_mpi_t s);
-gpg_err_code_t _gcry_ecc_gost_verify (gcry_mpi_t input, mpi_ec_t ec,
+gpg_err_code_t _gcry_ecc_gost_verify (gcry_mpi_t input, ECC_public_key *pkey,
                                       gcry_mpi_t r, gcry_mpi_t s);
 
 
-/*-- ecc-sm2.c --*/
-gpg_err_code_t _gcry_ecc_sm2_encrypt (gcry_sexp_t *r_ciph,
-                                      gcry_mpi_t input, mpi_ec_t ec);
-gpg_err_code_t _gcry_ecc_sm2_decrypt (gcry_sexp_t *r_plain,
-                                      gcry_sexp_t data_list, mpi_ec_t ec);
-gpg_err_code_t _gcry_ecc_sm2_sign (gcry_mpi_t input, mpi_ec_t ec,
-                                   gcry_mpi_t r, gcry_mpi_t s,
-                                   int flags, int hashalgo);
-gpg_err_code_t _gcry_ecc_sm2_verify (gcry_mpi_t input, mpi_ec_t ec,
-                                     gcry_mpi_t r, gcry_mpi_t s);
-
-
 #endif /*GCRY_ECC_COMMON_H*/
