@@ -33,12 +33,6 @@
 #include "pubkey-internal.h"
 
 
-/* Blinding is used to mitigate side-channel attacks.  You may undef
-   this to speed up the operation in case the system is secured
-   against physical and network mounted side-channel attacks.  */
-#define USE_BLINDING 1
-
-
 typedef struct
 {
   gcry_mpi_t p;	    /* prime */
@@ -67,8 +61,7 @@ static const char *elg_names[] =
 
 static int test_keys (ELG_secret_key *sk, unsigned int nbits, int nodie);
 static gcry_mpi_t gen_k (gcry_mpi_t p, int small_k);
-static gcry_err_code_t generate (ELG_secret_key *sk, unsigned nbits,
-                                 gcry_mpi_t **factors);
+static void generate (ELG_secret_key *sk, unsigned nbits, gcry_mpi_t **factors);
 static int  check_secret_key (ELG_secret_key *sk);
 static void do_encrypt (gcry_mpi_t a, gcry_mpi_t b, gcry_mpi_t input,
                         ELG_public_key *pkey);
@@ -275,10 +268,9 @@ gen_k( gcry_mpi_t p, int small_k )
  * Returns: 2 structures filled with all needed values
  *	    and an array with n-1 factors of (p-1)
  */
-static gcry_err_code_t
+static void
 generate ( ELG_secret_key *sk, unsigned int nbits, gcry_mpi_t **ret_factors )
 {
-  gcry_err_code_t rc;
   gcry_mpi_t p;    /* the prime */
   gcry_mpi_t p_min1;
   gcry_mpi_t g;
@@ -293,13 +285,7 @@ generate ( ELG_secret_key *sk, unsigned int nbits, gcry_mpi_t **ret_factors )
   if( qbits & 1 ) /* better have a even one */
     qbits++;
   g = mpi_alloc(1);
-  rc = _gcry_generate_elg_prime (0, nbits, qbits, g, &p, ret_factors);
-  if (rc)
-    {
-      mpi_free (p_min1);
-      mpi_free (g);
-      return rc;
-    }
+  p = _gcry_generate_elg_prime( 0, nbits, qbits, g, ret_factors );
   mpi_sub_ui(p_min1, p, 1);
 
 
@@ -373,8 +359,6 @@ generate ( ELG_secret_key *sk, unsigned int nbits, gcry_mpi_t **ret_factors )
 
   /* Now we can test our keys (this should never fail!) */
   test_keys ( sk, nbits - 64, 0 );
-
-  return 0;
 }
 
 
@@ -389,7 +373,6 @@ static gcry_err_code_t
 generate_using_x (ELG_secret_key *sk, unsigned int nbits, gcry_mpi_t x,
                   gcry_mpi_t **ret_factors )
 {
-  gcry_err_code_t rc;
   gcry_mpi_t p;      /* The prime.  */
   gcry_mpi_t p_min1; /* The prime minus 1.  */
   gcry_mpi_t g;      /* The generator.  */
@@ -412,13 +395,7 @@ generate_using_x (ELG_secret_key *sk, unsigned int nbits, gcry_mpi_t x,
   if ( (qbits & 1) ) /* Better have an even one.  */
     qbits++;
   g = mpi_alloc (1);
-  rc = _gcry_generate_elg_prime (0, nbits, qbits, g, &p, ret_factors );
-  if (rc)
-    {
-      mpi_free (p_min1);
-      mpi_free (g);
-      return rc;
-    }
+  p = _gcry_generate_elg_prime ( 0, nbits, qbits, g, ret_factors );
   mpi_sub_ui (p_min1, p, 1);
 
   if (DBG_CIPHER)
@@ -522,45 +499,15 @@ do_encrypt(gcry_mpi_t a, gcry_mpi_t b, gcry_mpi_t input, ELG_public_key *pkey )
 static void
 decrypt (gcry_mpi_t output, gcry_mpi_t a, gcry_mpi_t b, ELG_secret_key *skey )
 {
-  gcry_mpi_t t1, t2, r;
-  unsigned int nbits = mpi_get_nbits (skey->p);
+  gcry_mpi_t t1 = mpi_alloc_secure( mpi_get_nlimbs( skey->p ) );
 
   mpi_normalize (a);
   mpi_normalize (b);
 
-  t1 = mpi_snew (nbits);
-
-#ifdef USE_BLINDING
-
-  t2 = mpi_snew (nbits);
-  r  = mpi_new (nbits);
-
-  /* We need a random number of about the prime size.  The random
-     number merely needs to be unpredictable; thus we use level 0.  */
-  _gcry_mpi_randomize (r, nbits, GCRY_WEAK_RANDOM);
-
-  /* t1 = r^x mod p */
-  mpi_powm (t1, r, skey->x, skey->p);
-  /* t2 = (a * r)^-x mod p */
-  mpi_mulm (t2, a, r, skey->p);
-  mpi_powm (t2, t2, skey->x, skey->p);
-  mpi_invm (t2, t2, skey->p);
-  /* t1 = (t1 * t2) mod p*/
-  mpi_mulm (t1, t1, t2, skey->p);
-
-  mpi_free (r);
-  mpi_free (t2);
-
-#else /*!USE_BLINDING*/
-
   /* output = b/(a^x) mod p */
-  mpi_powm (t1, a, skey->x, skey->p);
-  mpi_invm (t1, t1, skey->p);
-
-#endif /*!USE_BLINDING*/
-
-  mpi_mulm (output, b, t1, skey->p);
-
+  mpi_powm( t1, a, skey->x, skey->p );
+  mpi_invm( t1, t1, skey->p );
+  mpi_mulm( output, b, t1, skey->p );
 #if 0
   if( DBG_CIPHER )
     {
@@ -571,7 +518,7 @@ decrypt (gcry_mpi_t output, gcry_mpi_t a, gcry_mpi_t b, ELG_secret_key *skey )
       log_mpidump ("elg decrypted M", output);
     }
 #endif
-  mpi_free (t1);
+  mpi_free(t1);
 }
 
 
@@ -715,7 +662,8 @@ elg_generate (const gcry_sexp_t genparms, gcry_sexp_t *r_skey)
     }
   else
     {
-      rc = generate (&sk, nbits, &factors);
+      generate (&sk, nbits, &factors);
+      rc = 0;
     }
   if (rc)
     goto leave;
