@@ -48,7 +48,6 @@
 #include "types.h"  /* for byte and u32 typedefs */
 #include "g10lib.h"
 #include "cipher.h"
-#include "cipher-internal.h"
 
 
 #define IDEA_KEYSIZE 16
@@ -153,7 +152,7 @@ invert_key( u16 *ek, u16 dk[IDEA_KEYLEN] )
     *--p = t2;
     *--p = t1;
     memcpy(dk, temp, sizeof(temp) );
-    wipememory(temp, sizeof(temp));
+    memset(temp, 0, sizeof(temp) );  /* burn temp */
 }
 
 
@@ -251,9 +250,7 @@ do_setkey( IDEA_context *c, const byte *key, unsigned int keylen )
     if( selftest_failed )
 	return GPG_ERR_SELFTEST_FAILED;
 
-    if (keylen != 16)
-      return GPG_ERR_INV_KEYLEN;
-
+    assert(keylen == 16);
     c->have_dk = 0;
     expand_key( key, c->ek );
     invert_key( c->ek, c->dk );
@@ -261,12 +258,10 @@ do_setkey( IDEA_context *c, const byte *key, unsigned int keylen )
 }
 
 static gcry_err_code_t
-idea_setkey (void *context, const byte *key, unsigned int keylen,
-             cipher_bulk_ops_t *bulk_ops)
+idea_setkey (void *context, const byte *key, unsigned int keylen)
 {
     IDEA_context *ctx = context;
     int rc = do_setkey (ctx, key, keylen);
-    (void)bulk_ops;
     _gcry_burn_stack (23+6*sizeof(void*));
     return rc;
 }
