@@ -5,7 +5,7 @@
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
+ * it under the terms of the GNU Lesser general Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
@@ -15,8 +15,8 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://www.gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #include <config.h>
@@ -29,6 +29,7 @@
 #include "g10lib.h"
 #include "mpi.h"
 #include "cipher.h"
+#include "ath.h"
 
 static gcry_mpi_t gen_prime (unsigned int nbits, int secret, int randomlevel,
                              int (*extra_check)(void *, gcry_mpi_t),
@@ -140,15 +141,18 @@ struct primepool_s
 };
 struct primepool_s *primepool;
 /* Mutex used to protect access to the primepool.  */
-GPGRT_LOCK_DEFINE (primepool_lock);
+static ath_mutex_t primepool_lock;
 
 
 gcry_err_code_t
 _gcry_primegen_init (void)
 {
-  /* This function was formerly used to initialize the primepool
-     Mutex. This has been replace by a static initialization.  */
-  return 0;
+  gcry_err_code_t ec;
+
+  ec = ath_mutex_init (&primepool_lock);
+  if (ec)
+    return gpg_err_code_from_errno (ec);
+  return ec;
 }
 
 
@@ -442,11 +446,12 @@ prime_generate_internal (int need_q_factor,
               goto leave;
             }
 
-          err = gpgrt_lock_lock (&primepool_lock);
-          if (err)
-            goto leave;
+          if (ath_mutex_lock (&primepool_lock))
+            {
+              err = GPG_ERR_INTERNAL;
+              goto leave;
+            }
           is_locked = 1;
-
           for (i = 0; i < n; i++)
             {
               perms[i] = 1;
@@ -465,9 +470,11 @@ prime_generate_internal (int need_q_factor,
                   pool[i] = get_pool_prime (fbits, poolrandomlevel);
                   if (!pool[i])
                     {
-                      err = gpgrt_lock_unlock (&primepool_lock);
-                      if (err)
-                        goto leave;
+                      if (ath_mutex_unlock (&primepool_lock))
+                        {
+                          err = GPG_ERR_INTERNAL;
+                          goto leave;
+                        }
                       is_locked = 0;
                     }
                 }
@@ -476,20 +483,23 @@ prime_generate_internal (int need_q_factor,
               pool_in_use[i] = i;
               factors[i] = pool[i];
             }
-
-          if (is_locked && (err = gpgrt_lock_unlock (&primepool_lock)))
-            goto leave;
+          if (is_locked && ath_mutex_unlock (&primepool_lock))
+            {
+              err = GPG_ERR_INTERNAL;
+              goto leave;
+            }
           is_locked = 0;
         }
       else
         {
           /* Get next permutation. */
           m_out_of_n ( (char*)perms, n, m);
-
-          if ((err = gpgrt_lock_lock (&primepool_lock)))
-            goto leave;
+          if (ath_mutex_lock (&primepool_lock))
+            {
+              err = GPG_ERR_INTERNAL;
+              goto leave;
+            }
           is_locked = 1;
-
           for (i = j = 0; (i < m) && (j < n); i++)
             if (perms[i])
               {
@@ -499,8 +509,11 @@ prime_generate_internal (int need_q_factor,
                     pool[i] = get_pool_prime (fbits, poolrandomlevel);
                     if (!pool[i])
                       {
-                        if ((err = gpgrt_lock_unlock (&primepool_lock)))
-                          goto leave;
+                        if (ath_mutex_unlock (&primepool_lock))
+                          {
+                            err = GPG_ERR_INTERNAL;
+                            goto leave;
+                          }
                         is_locked = 0;
                       }
                   }
@@ -509,11 +522,12 @@ prime_generate_internal (int need_q_factor,
                 pool_in_use[j] = i;
                 factors[j++] = pool[i];
               }
-
-          if (is_locked && (err = gpgrt_lock_unlock (&primepool_lock)))
-            goto leave;
+          if (is_locked && ath_mutex_unlock (&primepool_lock))
+            {
+              err = GPG_ERR_INTERNAL;
+              goto leave;
+            }
           is_locked = 0;
-
           if (i == n)
             {
               /* Ran out of permutations: Allocate new primes.  */
@@ -622,44 +636,47 @@ prime_generate_internal (int need_q_factor,
         }
     }
 
-  if (g && need_q_factor)
-    err = GPG_ERR_NOT_IMPLEMENTED;
-  else if (g)
+  if (g)
     {
       /* Create a generator (start with 3).  */
       gcry_mpi_t tmp = mpi_alloc (mpi_get_nlimbs (prime));
       gcry_mpi_t b = mpi_alloc (mpi_get_nlimbs (prime));
       gcry_mpi_t pmin1 = mpi_alloc (mpi_get_nlimbs (prime));
 
-      factors[n] = q;
-      factors[n + 1] = mpi_alloc_set_ui (2);
-      mpi_sub_ui (pmin1, prime, 1);
-      mpi_set_ui (g, 2);
-      do
+      if (need_q_factor)
+        err = GPG_ERR_NOT_IMPLEMENTED;
+      else
         {
-          mpi_add_ui (g, g, 1);
-          if (DBG_CIPHER)
-            log_printmpi ("checking g", g);
-          else
-            progress('^');
-          for (i = 0; i < n + 2; i++)
+          factors[n] = q;
+          factors[n + 1] = mpi_alloc_set_ui (2);
+          mpi_sub_ui (pmin1, prime, 1);
+          mpi_set_ui (g, 2);
+          do
             {
-              mpi_fdiv_q (tmp, pmin1, factors[i]);
-              /* No mpi_pow(), but it is okay to use this with mod
-                 prime.  */
-              mpi_powm (b, g, tmp, prime);
-              if (! mpi_cmp_ui (b, 1))
-                break;
+              mpi_add_ui (g, g, 1);
+              if (DBG_CIPHER)
+                log_printmpi ("checking g", g);
+              else
+                progress('^');
+              for (i = 0; i < n + 2; i++)
+                {
+                  mpi_fdiv_q (tmp, pmin1, factors[i]);
+                  /* No mpi_pow(), but it is okay to use this with mod
+                     prime.  */
+                  mpi_powm (b, g, tmp, prime);
+                  if (! mpi_cmp_ui (b, 1))
+                    break;
+                }
+              if (DBG_CIPHER)
+                progress('\n');
             }
-          if (DBG_CIPHER)
-            progress('\n');
-        }
-      while (i < n + 2);
+          while (i < n + 2);
 
-      mpi_free (factors[n+1]);
-      mpi_free (tmp);
-      mpi_free (b);
-      mpi_free (pmin1);
+          mpi_free (factors[n+1]);
+          mpi_free (tmp);
+          mpi_free (b);
+          mpi_free (pmin1);
+        }
     }
 
   if (! DBG_CIPHER)
@@ -669,7 +686,7 @@ prime_generate_internal (int need_q_factor,
  leave:
   if (pool)
     {
-      is_locked = !gpgrt_lock_lock (&primepool_lock);
+      is_locked = !ath_mutex_lock (&primepool_lock);
       for(i = 0; i < m; i++)
         {
           if (pool[i])
@@ -686,8 +703,8 @@ prime_generate_internal (int need_q_factor,
                 mpi_free (pool[i]);
             }
         }
-      if (is_locked)
-        err = gpgrt_lock_unlock (&primepool_lock);
+      if (is_locked && ath_mutex_unlock (&primepool_lock))
+        err = GPG_ERR_INTERNAL;
       is_locked = 0;
       xfree (pool);
     }
@@ -723,22 +740,19 @@ prime_generate_internal (int need_q_factor,
 
 
 /* Generate a prime used for discrete logarithm algorithms; i.e. this
-   prime will be public and no strong random is required.  On success
-   R_PRIME receives a new MPI with the prime.  On error R_PRIME is set
-   to NULL and an error code is returned.  If RET_FACTORS is not NULL
-   it is set to an allocated array of factors on success or to NULL on
-   error.  */
-gcry_err_code_t
+   prime will be public and no strong random is required.  */
+gcry_mpi_t
 _gcry_generate_elg_prime (int mode, unsigned pbits, unsigned qbits,
-			  gcry_mpi_t g,
-                          gcry_mpi_t *r_prime, gcry_mpi_t **ret_factors)
+			  gcry_mpi_t g, gcry_mpi_t **ret_factors)
 {
-  *r_prime = NULL;
-  if (ret_factors)
-    *ret_factors = NULL;
-  return prime_generate_internal ((mode == 1), r_prime, pbits, qbits, g,
-                                  ret_factors, GCRY_WEAK_RANDOM, 0, 0,
-                                  NULL, NULL);
+  gcry_mpi_t prime = NULL;
+
+  if (prime_generate_internal ((mode == 1), &prime, pbits, qbits, g,
+                               ret_factors, GCRY_WEAK_RANDOM, 0, 0,
+                               NULL, NULL))
+    prime = NULL; /* (Should be NULL in the error case anyway.)  */
+
+  return prime;
 }
 
 
@@ -752,16 +766,13 @@ gen_prime (unsigned int nbits, int secret, int randomlevel,
   unsigned int count1, count2;
   int *mods;
 
-  (void)count1; /* The value is not used, actually.  */
-
 /*   if (  DBG_CIPHER ) */
 /*     log_debug ("generate a prime of %u bits ", nbits ); */
 
   if (nbits < 16)
     log_fatal ("can't generate a prime with less than %d bits\n", 16);
 
-  mods = (secret? xmalloc_secure (no_of_small_prime_numbers * sizeof *mods)
-          /* */ : xmalloc (no_of_small_prime_numbers * sizeof *mods));
+  mods = xmalloc (no_of_small_prime_numbers * sizeof *mods);
   /* Make nbits fit into gcry_mpi_t implementation. */
   val_2  = mpi_alloc_set_ui( 2 );
   val_3 = mpi_alloc_set_ui( 3);
@@ -864,14 +875,11 @@ check_prime( gcry_mpi_t prime, gcry_mpi_t val_2, int rm_rounds,
   unsigned int x;
   unsigned int count=0;
 
-  if (prime->nlimbs && !(prime->d[0] & 1))
-    return 0;
-
   /* Check against small primes. */
   for (i=0; (x = small_prime_numbers[i]); i++ )
     {
       if ( mpi_divisible_ui( prime, x ) )
-        return !mpi_cmp_ui (prime, x);
+        return 0;
     }
 
   /* A quick Fermat test. */
@@ -941,25 +949,20 @@ is_prime (gcry_mpi_t n, int steps, unsigned int *count)
         }
       else
         {
-          /* We need to loop to avoid an X with value 0 or 1.  */
-          do
-            {
-              _gcry_mpi_randomize (x, nbits, GCRY_WEAK_RANDOM);
+          _gcry_mpi_randomize( x, nbits, GCRY_WEAK_RANDOM );
 
-              /* Make sure that the number is smaller than the prime
-               * and keep the randomness of the high bit. */
-              if (mpi_test_bit (x, nbits-2))
-                {
-                  mpi_set_highbit (x, nbits-2); /* Clear all higher bits. */
-                }
-              else
-                {
-                  mpi_set_highbit (x, nbits-2);
-                  mpi_clear_bit (x, nbits-2);
-                }
+          /* Make sure that the number is smaller than the prime and
+             keep the randomness of the high bit. */
+          if ( mpi_test_bit ( x, nbits-2) )
+            {
+              mpi_set_highbit ( x, nbits-2); /* Clear all higher bits. */
             }
-          while (mpi_cmp_ui (x, 1) <= 0);
-          gcry_assert (mpi_cmp (x, nminus1) < 0);
+          else
+            {
+              mpi_set_highbit( x, nbits-2 );
+              mpi_clear_bit( x, nbits-2 );
+            }
+          gcry_assert (mpi_cmp (x, nminus1) < 0 && mpi_cmp_ui (x, 1) > 0);
 	}
       mpi_powm ( y, x, q, n);
       if ( mpi_cmp_ui(y, 1) && mpi_cmp( y, nminus1 ) )
@@ -1177,43 +1180,21 @@ _gcry_prime_generate (gcry_mpi_t *prime, unsigned int prime_bits,
 gcry_err_code_t
 _gcry_prime_check (gcry_mpi_t x, unsigned int flags)
 {
-  (void)flags;
+  gcry_err_code_t rc = 0;
+  gcry_mpi_t val_2 = mpi_alloc_set_ui (2); /* Used by the Fermat test. */
 
-  switch (mpi_cmp_ui (x, 2))
-    {
-    case 0:  return 0;                /* 2 is a prime */
-    case -1: return GPG_ERR_NO_PRIME; /* Only numbers > 1 are primes.  */
-    }
+  (void)flags;
 
   /* We use 64 rounds because the prime we are going to test is not
      guaranteed to be a random one. */
-  if (check_prime (x, mpi_const (MPI_C_TWO), 64, NULL, NULL))
-    return 0;
+  if (! check_prime (x, val_2, 64, NULL, NULL))
+    rc = GPG_ERR_NO_PRIME;
 
-  return GPG_ERR_NO_PRIME;
+  mpi_free (val_2);
+
+  return rc;
 }
 
-
-/* Check whether the number X is prime according to FIPS 186-4 table C.2.  */
-gcry_err_code_t
-_gcry_fips186_4_prime_check (gcry_mpi_t x, unsigned int bits)
-{
-  gcry_err_code_t ec = GPG_ERR_NO_ERROR;
-
-  switch (mpi_cmp_ui (x, 2))
-    {
-    case 0:  return ec;               /* 2 is a prime */
-    case -1: return GPG_ERR_NO_PRIME; /* Only numbers > 1 are primes.  */
-    }
-
-  /* We use 5 or 4 rounds as specified in table C.2 */
-  if (! check_prime (x, mpi_const (MPI_C_TWO), bits > 1024 ? 4 : 5, NULL, NULL))
-    ec = GPG_ERR_NO_PRIME;
-
-  return ec;
-}
-
-
 /* Find a generator for PRIME where the factorization of (prime-1) is
    in the NULL terminated array FACTORS. Return the generator as a
    newly allocated MPI in R_G.  If START_G is not NULL, use this as s
@@ -1223,25 +1204,22 @@ _gcry_prime_group_generator (gcry_mpi_t *r_g,
                              gcry_mpi_t prime, gcry_mpi_t *factors,
                              gcry_mpi_t start_g)
 {
-  gcry_mpi_t tmp, b, pmin1, g;
-  int first, i, n;
+  gcry_mpi_t tmp   = mpi_new (0);
+  gcry_mpi_t b     = mpi_new (0);
+  gcry_mpi_t pmin1 = mpi_new (0);
+  gcry_mpi_t g = start_g? mpi_copy (start_g) : mpi_set_ui (NULL, 3);
+  int first = 1;
+  int i, n;
 
-  if (!r_g)
+  if (!factors || !r_g || !prime)
     return GPG_ERR_INV_ARG;
   *r_g = NULL;
-  if (!factors || !prime)
-    return GPG_ERR_INV_ARG;
 
   for (n=0; factors[n]; n++)
     ;
   if (n < 2)
     return GPG_ERR_INV_ARG;
 
-  tmp   = mpi_new (0);
-  b     = mpi_new (0);
-  pmin1 = mpi_new (0);
-  g     = start_g? mpi_copy (start_g) : mpi_set_ui (NULL, 3);
-
   /* Extra sanity check - usually disabled. */
 /*   mpi_set (tmp, factors[0]); */
 /*   for(i = 1; i < n; i++) */
@@ -1251,7 +1229,6 @@ _gcry_prime_group_generator (gcry_mpi_t *r_g,
 /*     return gpg_error (GPG_ERR_INV_ARG); */
 
   mpi_sub_ui (pmin1, prime, 1);
-  first = 1;
   do
     {
       if (first)
@@ -1312,7 +1289,7 @@ find_x931_prime (const gcry_mpi_t pfirst)
   mpi_set_bit (prime, 0);
 
   /* We use 64 Rabin-Miller rounds which is better and thus
-     sufficient.  We do not have a Lucas test implementation thus we
+     sufficient.  We do not have a Lucas test implementaion thus we
      can't do it in the X9.31 preferred way of running a few
      Rabin-Miller followed by one Lucas test.  */
   while ( !check_prime (prime, val_2, 64, NULL, NULL) )
@@ -1644,21 +1621,23 @@ _gcry_generate_fips186_2_prime (unsigned int pbits, unsigned int qbits,
 
 
 
-/* WARNING: The code below has not yet been tested!
- *
- * Generate the two prime used for DSA using the algorithm specified
- * in FIPS 186-3, A.1.1.2.  PBITS is the desired length of the prime P
- * and a QBITS the length of the prime Q.  If SEED is not supplied and
- * SEEDLEN is 0 the function generates an appropriate SEED.  On
- * success the generated primes are stored at R_Q and R_P, the counter
- * value is stored at R_COUNTER and the seed actually used for
- * generation is stored at R_SEED and R_SEEDVALUE.  The hash algorithm
- * used is stored at R_HASHALGO.
- *
- * Note that this function is very similar to the fips186_2 code.  Due
- * to the minor differences, other buffer sizes and for documentarion,
- * we use a separate function.
- */
+/* WARNING: The code below has not yet been tested!  However, it is
+   not yet used.  We need to wait for FIPS 186-3 final and for test
+   vectors.
+
+   Generate the two prime used for DSA using the algorithm specified
+   in FIPS 186-3, A.1.1.2.  PBITS is the desired length of the prime P
+   and a QBITS the length of the prime Q.  If SEED is not supplied and
+   SEEDLEN is 0 the function generates an appropriate SEED.  On
+   success the generated primes are stored at R_Q and R_P, the counter
+   value is stored at R_COUNTER and the seed actually used for
+   generation is stored at R_SEED and R_SEEDVALUE.  The hash algorithm
+   used is stored at R_HASHALGO.
+
+   Note that this function is very similar to the fips186_2 code.  Due
+   to the minor differences, other buffer sizes and for documentarion,
+   we use a separate function.
+*/
 gpg_err_code_t
 _gcry_generate_fips186_3_prime (unsigned int pbits, unsigned int qbits,
                                 const void *seed, size_t seedlen,
@@ -1670,7 +1649,7 @@ _gcry_generate_fips186_3_prime (unsigned int pbits, unsigned int qbits,
   gpg_err_code_t ec;
   unsigned char seed_help_buffer[256/8];  /* Used to hold a generated SEED. */
   unsigned char *seed_plus;     /* Malloced buffer to hold SEED+x.  */
-  unsigned char digest[256/8];  /* Helper buffer for SHA-2 digest.  */
+  unsigned char digest[256/8];  /* Helper buffer for SHA-1 digest.  */
   gcry_mpi_t val_2 = NULL;      /* Helper for the prime test.  */
   gcry_mpi_t tmpval = NULL;     /* Helper variable.  */
   int hashalgo;                 /* The id of the Approved Hash Function.  */
@@ -1689,7 +1668,9 @@ _gcry_generate_fips186_3_prime (unsigned int pbits, unsigned int qbits,
 
   /* Step 1:  Check the requested prime lengths.  */
   /* Note that due to the size of our buffers QBITS is limited to 256.  */
-  if (pbits == 2048 && qbits == 224)
+  if (pbits == 1024 && qbits == 160)
+    hashalgo = GCRY_MD_SHA1;
+  else if (pbits == 2048 && qbits == 224)
     hashalgo = GCRY_MD_SHA224;
   else if (pbits == 2048 && qbits == 256)
     hashalgo = GCRY_MD_SHA256;
@@ -1758,7 +1739,7 @@ _gcry_generate_fips186_3_prime (unsigned int pbits, unsigned int qbits,
         }
       _gcry_mpi_release (prime_q); prime_q = NULL;
       ec = _gcry_mpi_scan (&prime_q, GCRYMPI_FMT_USG,
-                           value_u, qbits/8, NULL);
+                           value_u, sizeof value_u, NULL);
       if (ec)
         goto leave;
       mpi_set_highbit (prime_q, qbits-1 );
@@ -1803,11 +1784,11 @@ _gcry_generate_fips186_3_prime (unsigned int pbits, unsigned int qbits,
               if (seed_plus[i])
                 break;
             }
-          _gcry_md_hash_buffer (hashalgo, digest, seed_plus, seedlen);
+          _gcry_md_hash_buffer (GCRY_MD_SHA1, digest, seed_plus, seedlen);
 
           _gcry_mpi_release (tmpval); tmpval = NULL;
           ec = _gcry_mpi_scan (&tmpval, GCRYMPI_FMT_USG,
-                               digest, qbits/8, NULL);
+                               digest, sizeof digest, NULL);
           if (ec)
             goto leave;
           if (value_j == value_n)
@@ -1843,12 +1824,11 @@ _gcry_generate_fips186_3_prime (unsigned int pbits, unsigned int qbits,
     }
 
   /* Step 12:  Save p, q, counter and seed.  */
-  /* log_debug ("fips186-3 pbits p=%u q=%u counter=%d\n", */
-  /*            mpi_get_nbits (prime_p), mpi_get_nbits (prime_q), counter); */
-  /* log_printhex ("fips186-3 seed", seed, seedlen); */
-  /* log_printmpi ("fips186-3    p", prime_p); */
-  /* log_printmpi ("fips186-3    q", prime_q); */
-
+  log_debug ("fips186-3 pbits p=%u q=%u counter=%d\n",
+             mpi_get_nbits (prime_p), mpi_get_nbits (prime_q), counter);
+  log_printhex ("fips186-3 seed", seed, seedlen);
+  log_printmpi ("fips186-3    p", prime_p);
+  log_printmpi ("fips186-3    q", prime_q);
   if (r_q)
     {
       *r_q = prime_q;
