@@ -1,7 +1,7 @@
 /* pubkey-util.c - Supporting functions for all pubkey modules.
  * Copyright (C) 1998, 1999, 2000, 2002, 2003, 2005,
  *               2007, 2008, 2011 Free Software Foundation, Inc.
- * Copyright (C) 2013, 2015 g10 Code GmbH
+ * Copyright (C) 2013  g10 Code GmbH
  *
  * This file is part of Libgcrypt.
  *
@@ -39,10 +39,9 @@ static int
 pss_verify_cmp (void *opaque, gcry_mpi_t tmp)
 {
   struct pk_encoding_ctx *ctx = opaque;
-  gcry_mpi_t value = ctx->verify_arg;
+  gcry_mpi_t hash = ctx->verify_arg;
 
-  return _gcry_rsa_pss_verify (value, !(ctx->flags & PUBKEY_FLAG_PREHASH),
-                               tmp, ctx->nbits - 1,
+  return _gcry_rsa_pss_verify (hash, tmp, ctx->nbits - 1,
                                ctx->hash_algo, ctx->saltlen);
 }
 
@@ -82,11 +81,6 @@ _gcry_pk_util_parse_flaglist (gcry_sexp_t list,
               encoding = PUBKEY_ENC_RAW;
               flags |= PUBKEY_FLAG_RAW_FLAG; /* Explicitly given.  */
             }
-          else if (!memcmp (s, "sm2", 3))
-            {
-                encoding = PUBKEY_ENC_RAW;
-                flags |= PUBKEY_FLAG_SM2 | PUBKEY_FLAG_RAW_FLAG;
-            }
           else if (!igninvflag)
             rc = GPG_ERR_INV_FLAG;
           break;
@@ -113,7 +107,6 @@ _gcry_pk_util_parse_flaglist (gcry_sexp_t list,
             {
               encoding = PUBKEY_ENC_RAW;
               flags |= PUBKEY_FLAG_EDDSA;
-              flags |= PUBKEY_FLAG_DJB_TWEAK;
             }
           else if (!memcmp (s, "pkcs1", 5) && encoding == PUBKEY_ENC_UNKNOWN)
             {
@@ -138,8 +131,6 @@ _gcry_pk_util_parse_flaglist (gcry_sexp_t list,
             flags |= PUBKEY_FLAG_RFC6979;
           else if (!memcmp (s, "noparam", 7))
             ; /* Ignore - it is the default.  */
-          else if (!memcmp (s, "prehash", 7))
-            flags |= PUBKEY_FLAG_PREHASH;
           else if (!igninvflag)
             rc = GPG_ERR_INV_FLAG;
           break;
@@ -151,30 +142,9 @@ _gcry_pk_util_parse_flaglist (gcry_sexp_t list,
             rc = GPG_ERR_INV_FLAG;
           break;
 
-        case 9:
-          if (!memcmp (s, "pkcs1-raw", 9) && encoding == PUBKEY_ENC_UNKNOWN)
-            {
-              encoding = PUBKEY_ENC_PKCS1_RAW;
-              flags |= PUBKEY_FLAG_FIXEDLEN;
-            }
-          else if (!memcmp (s, "djb-tweak", 9))
-            {
-              encoding = PUBKEY_ENC_RAW;
-              flags |= PUBKEY_FLAG_DJB_TWEAK;
-            }
-          else if (!memcmp (s, "no-prefix", 9))
-            flags |= PUBKEY_FLAG_NO_PREFIX;
-          else if (!igninvflag)
-            rc = GPG_ERR_INV_FLAG;
-          break;
-
         case 10:
           if (!memcmp (s, "igninvflag", 10))
             igninvflag = 1;
-          else if (!memcmp (s, "no-keytest", 10))
-            flags |= PUBKEY_FLAG_NO_KEYTEST;
-          else if (!igninvflag)
-            rc = GPG_ERR_INV_FLAG;
           break;
 
         case 11:
@@ -202,14 +172,6 @@ _gcry_pk_util_parse_flaglist (gcry_sexp_t list,
         }
     }
 
-  if (fips_mode () && igninvflag)
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK_FLAGS))
-        rc = GPG_ERR_INV_FLAG;
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
   if (r_flags)
     *r_flags = flags;
   if (r_encoding)
@@ -235,15 +197,6 @@ get_hash_algo (const char *s, size_t n)
     { "md4",    GCRY_MD_MD4 },
     { "tiger",  GCRY_MD_TIGER },
     { "haval",  GCRY_MD_HAVAL },
-    { "sha3-224", GCRY_MD_SHA3_224 },
-    { "sha3-256", GCRY_MD_SHA3_256 },
-    { "sha3-384", GCRY_MD_SHA3_384 },
-    { "sha3-512", GCRY_MD_SHA3_512 },
-    { "sm3", GCRY_MD_SM3 },
-    { "shake128", GCRY_MD_SHAKE128 },
-    { "shake256", GCRY_MD_SHAKE256 },
-    { "sha512-224", GCRY_MD_SHA512_224 },
-    { "sha512-256", GCRY_MD_SHA512_256 },
     { NULL, 0 }
   };
   int algo;
@@ -261,7 +214,7 @@ get_hash_algo (const char *s, size_t n)
     {
       /* In case of not listed or dynamically allocated hash
 	 algorithm we fall back to this somewhat slower
-	 method.  Further, it also allows using OIDs as
+	 method.  Further, it also allows to use OIDs as
 	 algorithm names. */
       char *tmpname;
 
@@ -451,8 +404,6 @@ _gcry_pk_util_preparse_sigval (gcry_sexp_t s_sig, const char **algo_names,
         *r_eccflags = PUBKEY_FLAG_EDDSA;
       if (!strcmp (name, "gost"))
         *r_eccflags = PUBKEY_FLAG_GOST;
-      if (!strcmp (name, "sm2"))
-        *r_eccflags = PUBKEY_FLAG_SM2;
     }
 
   *r_parms = l2;
@@ -642,18 +593,9 @@ _gcry_pk_util_init_encoding_ctx (struct pk_encoding_ctx *ctx,
   ctx->nbits = nbits;
   ctx->encoding = PUBKEY_ENC_UNKNOWN;
   ctx->flags = 0;
-  if (fips_mode ())
-    {
-      ctx->hash_algo = GCRY_MD_SHA256;
-    }
-  else
-    {
-      ctx->hash_algo = GCRY_MD_SHA1;
-    }
+  ctx->hash_algo = GCRY_MD_SHA1;
   ctx->label = NULL;
   ctx->labellen = 0;
-  ctx->rnd = NULL;
-  ctx->rndlen = 0;
   ctx->saltlen = 20;
   ctx->verify_cmp = NULL;
   ctx->verify_arg = NULL;
@@ -664,21 +606,19 @@ void
 _gcry_pk_util_free_encoding_ctx (struct pk_encoding_ctx *ctx)
 {
   xfree (ctx->label);
-  xfree (ctx->rnd);
 }
 
 
 /* Take the hash value and convert into an MPI, suitable for
    passing to the low level functions.  We currently support the
    old style way of passing just a MPI and the modern interface which
-   allows passing flags so that we can choose between raw and pkcs1
+   allows to pass flags so that we can choose between raw and pkcs1
    padding - may be more padding options later.
 
    (<mpi>)
    or
    (data
-    [(flags [raw, direct, pkcs1, oaep, pss,
-             no-blinding, rfc6979, eddsa, prehash])]
+    [(flags [raw, direct, pkcs1, oaep, pss, no-blinding, rfc6979, eddsa])]
     [(hash <algo> <value>)]
     [(value <text>)]
     [(hash-algo <algo>)]
@@ -690,11 +630,11 @@ _gcry_pk_util_free_encoding_ctx (struct pk_encoding_ctx *ctx)
    Either the VALUE or the HASH element must be present for use
    with signatures.  VALUE is used for encryption.
 
-   HASH-ALGO is specific to OAEP, PSS and EDDSA.
+   HASH-ALGO is specific to OAEP and EDDSA.
 
-   LABEL is used for OAEP, RAW or RFC6979.
+   LABEL is specific to OAEP.
 
-   SALT-LENGTH is for PSS, it is limited to 16384 bytes.
+   SALT-LENGTH is for PSS.
 
    RANDOM-OVERRIDE is used to replace random nonces for regression
    testing.  */
@@ -713,10 +653,7 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
   ldata = sexp_find_token (input, "data", 0);
   if (!ldata)
     { /* assume old style */
-      int mpifmt = (ctx->flags & PUBKEY_FLAG_RAW_FLAG) ?
-        GCRYMPI_FMT_OPAQUE : GCRYMPI_FMT_STD;
-
-      *ret_mpi = sexp_nth_mpi (input, 0, mpifmt);
+      *ret_mpi = sexp_nth_mpi (input, 0, 0);
       return *ret_mpi ? GPG_ERR_NO_ERROR : GPG_ERR_INV_OBJ;
     }
 
@@ -744,48 +681,7 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
   else if (unknown_flag)
     rc = GPG_ERR_INV_FLAG;
   else if (ctx->encoding == PUBKEY_ENC_RAW
-           && (ctx->flags & PUBKEY_FLAG_BYTE_STRING))
-    {
-      gcry_sexp_t list;
-      void *value;
-      size_t valuelen;
-
-      if (!lvalue)
-        {
-          rc = GPG_ERR_INV_OBJ;
-          goto leave;
-        }
-
-      /* Get optional LABEL. */
-      list = sexp_find_token (ldata, "label", 0);
-      if (list)
-        {
-          ctx->label = sexp_nth_buffer (list, 1, &ctx->labellen);
-          sexp_release (list);
-        }
-
-      /* Get optional RANDOM-OVERRIDE. */
-      list = sexp_find_token (ldata, "random-override", 0);
-      if (list)
-        {
-          ctx->rnd = sexp_nth_buffer (list, 1, &ctx->rndlen);
-          sexp_release (list);
-        }
-
-      /* Get VALUE.  */
-      value = sexp_nth_buffer (lvalue, 1, &valuelen);
-      if (!value)
-        {
-          rc = GPG_ERR_INV_OBJ;
-          goto leave;
-        }
-
-      /* Note that mpi_set_opaque takes ownership of VALUE.  */
-      *ret_mpi = mpi_set_opaque (NULL, value, valuelen*8);
-    }
-  else if (ctx->encoding == PUBKEY_ENC_RAW
-           && ((parsed_flags & PUBKEY_FLAG_EDDSA)
-               || (ctx->flags & PUBKEY_FLAG_EDDSA)))
+           && (parsed_flags & PUBKEY_FLAG_EDDSA))
     {
       /* Prepare for EdDSA.  */
       gcry_sexp_t list;
@@ -797,7 +693,6 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
           rc = GPG_ERR_INV_OBJ;
           goto leave;
         }
-      /* Hash algo is determined by curve.  No hash-algo is OK.  */
       /* Get HASH-ALGO. */
       list = sexp_find_token (ldata, "hash-algo", 0);
       if (list)
@@ -813,32 +708,11 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
             }
           sexp_release (list);
         }
+      else
+        rc = GPG_ERR_INV_OBJ;
       if (rc)
         goto leave;
 
-      /* Get LABEL. */
-      list = sexp_find_token (ldata, "label", 0);
-      if (list)
-        {
-          s = sexp_nth_data (list, 1, &n);
-          if (!s)
-            rc = GPG_ERR_NO_OBJ;
-          else if (n > 0)
-            {
-              ctx->label = xtrymalloc (n);
-              if (!ctx->label)
-                rc = gpg_err_code_from_syserror ();
-              else
-                {
-                  memcpy (ctx->label, s, n);
-                  ctx->labellen = n;
-                }
-            }
-          sexp_release (list);
-          if (rc)
-            goto leave;
-        }
-
       /* Get VALUE.  */
       value = sexp_nth_buffer (lvalue, 1, &valuelen);
       if (!value)
@@ -862,66 +736,27 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
       /* Note that mpi_set_opaque takes ownership of VALUE.  */
       *ret_mpi = mpi_set_opaque (NULL, value, valuelen*8);
     }
-  else if (ctx->encoding == PUBKEY_ENC_RAW
-           && (lhash || (lvalue && (parsed_flags & PUBKEY_FLAG_PREHASH)))
+  else if (ctx->encoding == PUBKEY_ENC_RAW && lhash
            && ((parsed_flags & PUBKEY_FLAG_RAW_FLAG)
                || (parsed_flags & PUBKEY_FLAG_RFC6979)))
     {
-      void * value;
-      size_t valuelen;
-      gcry_sexp_t list;
-
       /* Raw encoding along with a hash element.  This is commonly
          used for DSA.  For better backward error compatibility we
          allow this only if either the rfc6979 flag has been given or
          the raw flags was explicitly given.  */
-
-      if (lvalue && (parsed_flags & PUBKEY_FLAG_PREHASH))
+      if (sexp_length (lhash) != 3)
+        rc = GPG_ERR_INV_OBJ;
+      else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
+        rc = GPG_ERR_INV_OBJ;
+      else
         {
-          /* Get HASH-ALGO. */
-          list = sexp_find_token (ldata, "hash-algo", 0);
-          if (list)
-            {
-              s = sexp_nth_data (list, 1, &n);
-              if (!s)
-                rc = GPG_ERR_NO_OBJ;
-              else
-                {
-                  ctx->hash_algo = get_hash_algo (s, n);
-                  if (!ctx->hash_algo)
-                    rc = GPG_ERR_DIGEST_ALGO;
-                }
-              sexp_release (list);
-              if (rc)
-                goto leave;
-            }
+          void *value;
+          size_t valuelen;
 
-          /* Get optional LABEL.  */
-          list = sexp_find_token (ldata, "label", 0);
-          if (list)
-            {
-              s = sexp_nth_data (list, 1, &n);
-              if (!s)
-                rc = GPG_ERR_NO_OBJ;
-              else if (n > 0)
-                {
-                  ctx->label = xtrymalloc (n);
-                  if (!ctx->label)
-                    rc = gpg_err_code_from_syserror ();
-                  else
-                    {
-                      memcpy (ctx->label, s, n);
-                      ctx->labellen = n;
-                    }
-                }
-              else
-                rc = GPG_ERR_INV_ARG;
-              sexp_release (list);
-              if (rc)
-                goto leave;
-            }
-
-          if ( !(value=sexp_nth_buffer (lvalue, 1, &valuelen)) || !valuelen )
+	  ctx->hash_algo = get_hash_algo (s, n);
+          if (!ctx->hash_algo)
+            rc = GPG_ERR_DIGEST_ALGO;
+          else if (!(value=sexp_nth_buffer (lhash, 2, &valuelen)))
             rc = GPG_ERR_INV_OBJ;
           else if ((valuelen * 8) < valuelen)
             {
@@ -931,64 +766,10 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
           else
             *ret_mpi = mpi_set_opaque (NULL, value, valuelen*8);
         }
-      else if (lhash)
-        {
-          /* Get optional LABEL.  */
-          list = sexp_find_token (ldata, "label", 0);
-          if (list)
-            {
-              s = sexp_nth_data (list, 1, &n);
-              if (!s)
-                rc = GPG_ERR_NO_OBJ;
-              else if (n > 0)
-                {
-                  ctx->label = xtrymalloc (n);
-                  if (!ctx->label)
-                    rc = gpg_err_code_from_syserror ();
-                  else
-                    {
-                      memcpy (ctx->label, s, n);
-                      ctx->labellen = n;
-                    }
-                }
-              else
-                rc = GPG_ERR_INV_ARG;
-              sexp_release (list);
-              if (rc)
-                goto leave;
-            }
-
-          if (sexp_length (lhash) != 3)
-            rc = GPG_ERR_INV_OBJ;
-          else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
-            rc = GPG_ERR_INV_OBJ;
-          else
-            {
-              ctx->hash_algo = get_hash_algo (s, n);
-
-              if (!ctx->hash_algo)
-                rc = GPG_ERR_DIGEST_ALGO;
-              else if ( !(value=sexp_nth_buffer (lhash, 2, &valuelen))
-                        || !valuelen )
-                rc = GPG_ERR_INV_OBJ;
-              else if ((valuelen * 8) < valuelen)
-                {
-                  xfree (value);
-                  rc = GPG_ERR_TOO_LARGE;
-                }
-              else
-                *ret_mpi = mpi_set_opaque (NULL, value, valuelen*8);
-            }
-        }
-      else
-        rc = GPG_ERR_CONFLICT;
-
-      if (rc)
-        goto leave;
     }
   else if (ctx->encoding == PUBKEY_ENC_RAW && lvalue)
     {
-      /* RFC6979 may only be used with the a hash value and not the
+      /* RFC6969 may only be used with the a hash value and not the
          MPI based value.  */
       if (parsed_flags & PUBKEY_FLAG_RFC6979)
         {
@@ -1018,17 +799,6 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
           list = sexp_find_token (ldata, "random-override", 0);
           if (list)
             {
-              if (fips_mode ())
-                {
-                  if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK_FLAGS))
-                    {
-                      sexp_release (list);
-                      rc = GPG_ERR_INV_FLAG;
-                      goto leave;
-                    }
-                  else
-                    fips_service_indicator_mark_non_compliant ();
-                }
               s = sexp_nth_data (list, 1, &n);
               if (!s)
                 rc = GPG_ERR_NO_OBJ;
@@ -1055,97 +825,30 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
           xfree (random_override);
         }
     }
-  else if (ctx->encoding == PUBKEY_ENC_PKCS1
-           && (lhash || (lvalue && (parsed_flags & PUBKEY_FLAG_PREHASH)))
+  else if (ctx->encoding == PUBKEY_ENC_PKCS1 && lhash
 	   && (ctx->op == PUBKEY_OP_SIGN || ctx->op == PUBKEY_OP_VERIFY))
     {
-      if (lvalue && (parsed_flags & PUBKEY_FLAG_PREHASH))
-        {
-          void * value;
-          size_t valuelen;
-          gcry_sexp_t list;
-
-          /* Get HASH-ALGO. */
-          list = sexp_find_token (ldata, "hash-algo", 0);
-          if (list)
-            {
-              s = sexp_nth_data (list, 1, &n);
-              if (!s)
-                rc = GPG_ERR_NO_OBJ;
-              else
-                {
-                  ctx->hash_algo = get_hash_algo (s, n);
-                  if (!ctx->hash_algo)
-                    rc = GPG_ERR_DIGEST_ALGO;
-                }
-              sexp_release (list);
-              if (rc)
-                goto leave;
-            }
-
-          if ( !(value=sexp_nth_buffer (lvalue, 1, &valuelen)) || !valuelen )
-            rc = GPG_ERR_INV_OBJ;
-          else if ((valuelen * 8) < valuelen)
-            {
-              xfree (value);
-              rc = GPG_ERR_TOO_LARGE;
-            }
-          else
-            {
-              void *hash;
-
-              n = _gcry_md_get_algo_dlen (ctx->hash_algo);
-              hash = xtrymalloc (n);
-              if (!hash)
-                rc = gpg_err_code_from_syserror ();
-              else
-                {
-                  _gcry_md_hash_buffer (ctx->hash_algo, hash, value, valuelen);
-                  rc = _gcry_rsa_pkcs1_encode_for_sig (ret_mpi, ctx->nbits,
-                                                       hash, n, ctx->hash_algo);
-                  xfree (hash);
-                }
-            }
-        }
-      else if (lhash)
-        {
-          if (sexp_length (lhash) != 3)
-            rc = GPG_ERR_INV_OBJ;
-          else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
-            rc = GPG_ERR_INV_OBJ;
-          else
-            {
-              const void * value;
-              size_t valuelen;
-
-              ctx->hash_algo = get_hash_algo (s, n);
-
-              if (!ctx->hash_algo)
-                rc = GPG_ERR_DIGEST_ALGO;
-              else if ( !(value=sexp_nth_data (lhash, 2, &valuelen))
-                        || !valuelen )
-                rc = GPG_ERR_INV_OBJ;
-              else
-                rc = _gcry_rsa_pkcs1_encode_for_sig (ret_mpi, ctx->nbits,
-                                                     value, valuelen,
-                                                     ctx->hash_algo);
-            }
-        }
-    }
-  else if (ctx->encoding == PUBKEY_ENC_PKCS1_RAW && lvalue
-	   && (ctx->op == PUBKEY_OP_SIGN || ctx->op == PUBKEY_OP_VERIFY))
-    {
-      const void * value;
-      size_t valuelen;
-
-      if (sexp_length (lvalue) != 2)
+      if (sexp_length (lhash) != 3)
         rc = GPG_ERR_INV_OBJ;
-      else if ( !(value=sexp_nth_data (lvalue, 1, &valuelen))
-                || !valuelen )
+      else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
         rc = GPG_ERR_INV_OBJ;
       else
-        rc = _gcry_rsa_pkcs1_encode_raw_for_sig (ret_mpi, ctx->nbits,
-                                                 value, valuelen);
+        {
+          const void * value;
+          size_t valuelen;
+
+	  ctx->hash_algo = get_hash_algo (s, n);
+
+          if (!ctx->hash_algo)
+            rc = GPG_ERR_DIGEST_ALGO;
+          else if ( !(value=sexp_nth_data (lhash, 2, &valuelen))
+                    || !valuelen )
+            rc = GPG_ERR_INV_OBJ;
+          else
+	    rc = _gcry_rsa_pkcs1_encode_for_sig (ret_mpi, ctx->nbits,
+                                                 value, valuelen,
+                                                 ctx->hash_algo);
+        }
     }
   else if (ctx->encoding == PUBKEY_ENC_OAEP && lvalue
 	   && ctx->op == PUBKEY_OP_ENCRYPT)
@@ -1205,17 +908,6 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
           list = sexp_find_token (ldata, "random-override", 0);
           if (list)
             {
-              if (fips_mode ())
-                {
-                  if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK_FLAGS))
-                    {
-                      sexp_release (list);
-                      rc = GPG_ERR_INV_FLAG;
-                      goto leave;
-                    }
-                  else
-                    fips_service_indicator_mark_non_compliant ();
-                }
               s = sexp_nth_data (list, 1, &n);
               if (!s)
                 rc = GPG_ERR_NO_OBJ;
@@ -1243,191 +935,100 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
           xfree (random_override);
 	}
     }
-  else if (ctx->encoding == PUBKEY_ENC_PSS && ctx->op == PUBKEY_OP_SIGN)
+  else if (ctx->encoding == PUBKEY_ENC_PSS && lhash
+	   && ctx->op == PUBKEY_OP_SIGN)
     {
-      const void * value;
-      size_t valuelen;
-      gcry_sexp_t list;
-      void *random_override = NULL;
-
-      if (lvalue)
+      if (sexp_length (lhash) != 3)
+        rc = GPG_ERR_INV_OBJ;
+      else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
+        rc = GPG_ERR_INV_OBJ;
+      else
         {
-          /* Get HASH-ALGO. */
-          list = sexp_find_token (ldata, "hash-algo", 0);
-          if (list)
-            {
-              s = sexp_nth_data (list, 1, &n);
-              if (!s)
-                rc = GPG_ERR_NO_OBJ;
-              else
-                {
-                  ctx->hash_algo = get_hash_algo (s, n);
-                  if (!ctx->hash_algo)
-                    rc = GPG_ERR_DIGEST_ALGO;
-                }
-              sexp_release (list);
-              if (rc)
-                goto leave;
-            }
+          const void * value;
+          size_t valuelen;
+          void *random_override = NULL;
+          size_t random_override_len = 0;
 
-          if ( !(value=sexp_nth_data (lvalue, 1, &valuelen)) || !valuelen )
-            rc = GPG_ERR_INV_OBJ;
-          ctx->flags |= PUBKEY_FLAG_PREHASH;
-        }
-      else if (lhash)
-        {
-          if (sexp_length (lhash) != 3)
-            rc = GPG_ERR_INV_OBJ;
-          else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
+	  ctx->hash_algo = get_hash_algo (s, n);
+
+          if (!ctx->hash_algo)
+            rc = GPG_ERR_DIGEST_ALGO;
+          else if ( !(value=sexp_nth_data (lhash, 2, &valuelen))
+                    || !valuelen )
             rc = GPG_ERR_INV_OBJ;
           else
-            {
-              ctx->hash_algo = get_hash_algo (s, n);
+	    {
+	      gcry_sexp_t list;
 
-              if (!ctx->hash_algo)
-                rc = GPG_ERR_DIGEST_ALGO;
-              else if ( !(value=sexp_nth_data (lhash, 2, &valuelen))
-                        || !valuelen )
-                rc = GPG_ERR_INV_OBJ;
-            }
-        }
-      else
-        rc = GPG_ERR_CONFLICT;
+	      /* Get SALT-LENGTH. */
+	      list = sexp_find_token (ldata, "salt-length", 0);
+	      if (list)
+		{
+		  s = sexp_nth_data (list, 1, &n);
+		  if (!s)
+		    {
+		      rc = GPG_ERR_NO_OBJ;
+		      goto leave;
+		    }
+		  ctx->saltlen = (unsigned int)strtoul (s, NULL, 10);
+		  sexp_release (list);
+		}
 
-      if (rc)
-        goto leave;
-
-      /* Get SALT-LENGTH. */
-      list = sexp_find_token (ldata, "salt-length", 0);
-      if (list)
-        {
-          s = sexp_nth_data (list, 1, &n);
-          if (!s)
-            {
-              rc = GPG_ERR_NO_OBJ;
-              goto leave;
-            }
-          ctx->saltlen = (unsigned int)strtoul (s, NULL, 10);
-          sexp_release (list);
-        }
-
-      /* Get optional RANDOM-OVERRIDE.  */
-      list = sexp_find_token (ldata, "random-override", 0);
-      if (list)
-        {
-          if (fips_mode ())
-            {
-              if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK_FLAGS))
+              /* Get optional RANDOM-OVERRIDE.  */
+              list = sexp_find_token (ldata, "random-override", 0);
+              if (list)
                 {
+                  s = sexp_nth_data (list, 1, &n);
+                  if (!s)
+                    rc = GPG_ERR_NO_OBJ;
+                  else if (n > 0)
+                    {
+                      random_override = xtrymalloc (n);
+                      if (!random_override)
+                        rc = gpg_err_code_from_syserror ();
+                      else
+                        {
+                          memcpy (random_override, s, n);
+                          random_override_len = n;
+                        }
+                    }
                   sexp_release (list);
-                  rc = GPG_ERR_INV_FLAG;
-                  goto leave;
+                  if (rc)
+                    goto leave;
                 }
-              else
-                fips_service_indicator_mark_non_compliant ();
-            }
-          s = sexp_nth_data (list, 1, &n);
-          if (!s)
-            rc = GPG_ERR_NO_OBJ;
-          else if (n == ctx->saltlen)
-            {
-              random_override = xtrymalloc (n);
-              if (!random_override)
-                rc = gpg_err_code_from_syserror ();
-              else
-                memcpy (random_override, s, n);
-            }
-          else
-            rc = GPG_ERR_INV_ARG;
-          sexp_release (list);
-          if (rc)
-            goto leave;
-        }
 
-      /* Encode the data.  (NBITS-1 is due to 8.1.1, step 1.) */
-      rc = _gcry_rsa_pss_encode (ret_mpi, ctx->nbits - 1,
-                                 ctx->hash_algo, ctx->saltlen,
-                                 !(ctx->flags & PUBKEY_FLAG_PREHASH),
-                                 value, valuelen,
-                                 random_override);
-      xfree (random_override);
+              /* Encode the data.  (NBITS-1 is due to 8.1.1, step 1.) */
+	      rc = _gcry_rsa_pss_encode (ret_mpi, ctx->nbits - 1,
+                                         ctx->hash_algo,
+                                         value, valuelen, ctx->saltlen,
+                                         random_override, random_override_len);
+
+              xfree (random_override);
+	    }
+        }
     }
-  else if (ctx->encoding == PUBKEY_ENC_PSS && ctx->op == PUBKEY_OP_VERIFY)
+  else if (ctx->encoding == PUBKEY_ENC_PSS && lhash
+	   && ctx->op == PUBKEY_OP_VERIFY)
     {
-      gcry_sexp_t list;
-
-      if (lvalue)
-        {
-          /* Get HASH-ALGO. */
-          list = sexp_find_token (ldata, "hash-algo", 0);
-          if (list)
-            {
-              s = sexp_nth_data (list, 1, &n);
-              if (!s)
-                rc = GPG_ERR_NO_OBJ;
-              else
-                {
-                  ctx->hash_algo = get_hash_algo (s, n);
-                  if (!ctx->hash_algo)
-                    rc = GPG_ERR_DIGEST_ALGO;
-                }
-              sexp_release (list);
-              if (rc)
-                goto leave;
-            }
-
-          *ret_mpi = sexp_nth_mpi (lvalue, 1, GCRYMPI_FMT_OPAQUE);
-          if (!*ret_mpi)
-            rc = GPG_ERR_INV_OBJ;
-
-          ctx->flags |= PUBKEY_FLAG_PREHASH;
-        }
-      else if (lhash)
-        {
-          if (sexp_length (lhash) != 3)
-            rc = GPG_ERR_INV_OBJ;
-          else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
-            rc = GPG_ERR_INV_OBJ;
-          else
-            {
-              ctx->hash_algo = get_hash_algo (s, n);
-
-              if (!ctx->hash_algo)
-                rc = GPG_ERR_DIGEST_ALGO;
-              else
-                {
-                  *ret_mpi = sexp_nth_mpi (lhash, 2, GCRYMPI_FMT_OPAQUE);
-                  if (!*ret_mpi)
-                    rc = GPG_ERR_INV_OBJ;
-                }
-            }
-        }
+      if (sexp_length (lhash) != 3)
+        rc = GPG_ERR_INV_OBJ;
+      else if ( !(s=sexp_nth_data (lhash, 1, &n)) || !n )
+        rc = GPG_ERR_INV_OBJ;
       else
-        rc = GPG_ERR_CONFLICT;
-
-      if (rc)
-        goto leave;
-
-      /* Get SALT-LENGTH. */
-      list = sexp_find_token (ldata, "salt-length", 0);
-      if (list)
         {
-          s = sexp_nth_data (list, 1, &n);
-          if (!s)
-            {
-              rc = GPG_ERR_NO_OBJ;
-              goto leave;
-            }
-          ctx->saltlen = (unsigned int)strtoul (s, NULL, 10);
-          if (ctx->saltlen > 16384)
-            rc = GPG_ERR_TOO_LARGE;
-          sexp_release (list);
-          if (rc)
-            goto leave;
-        }
+	  ctx->hash_algo = get_hash_algo (s, n);
 
-      ctx->verify_cmp = pss_verify_cmp;
-      ctx->verify_arg = *ret_mpi;
+          if (!ctx->hash_algo)
+            rc = GPG_ERR_DIGEST_ALGO;
+	  else
+	    {
+	      *ret_mpi = sexp_nth_mpi (lhash, 2, GCRYMPI_FMT_USG);
+	      if (!*ret_mpi)
+		rc = GPG_ERR_INV_OBJ;
+	      ctx->verify_cmp = pss_verify_cmp;
+	      ctx->verify_arg = *ret_mpi;
+	    }
+	}
     }
   else
     rc = GPG_ERR_CONFLICT;
@@ -1438,13 +1039,11 @@ _gcry_pk_util_data_to_mpi (gcry_sexp_t input, gcry_mpi_t *ret_mpi,
   sexp_release (lvalue);
 
   if (!rc)
-    ctx->flags |= parsed_flags;
+    ctx->flags = parsed_flags;
   else
     {
       xfree (ctx->label);
       ctx->label = NULL;
-      xfree (ctx->rnd);
-      ctx->rnd = NULL;
     }
 
   return rc;
