@@ -6,7 +6,7 @@
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
+ * it under the terms of the GNU Lesser general Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
@@ -28,13 +28,14 @@
 #include "g10lib.h"
 #include "mpi.h"
 #include "cipher.h"
+#include "ath.h"
 #include "context.h"
 #include "pubkey-internal.h"
 
 
 /* This is the list of the public-key algorithms included in
    Libgcrypt.  */
-static gcry_pk_spec_t * const pubkey_list[] =
+static gcry_pk_spec_t *pubkey_list[] =
   {
 #if USE_ECC
     &_gcry_pubkey_spec_ecc,
@@ -48,7 +49,6 @@ static gcry_pk_spec_t * const pubkey_list[] =
 #if USE_ELGAMAL
     &_gcry_pubkey_spec_elg,
 #endif
-    &_gcry_pubkey_spec_kem,
     NULL
   };
 
@@ -58,17 +58,18 @@ map_algo (int algo)
 {
  switch (algo)
    {
-   case GCRY_PK_RSA_E: return GCRY_PK_RSA;
-   case GCRY_PK_RSA_S: return GCRY_PK_RSA;
-   case GCRY_PK_ELG_E: return GCRY_PK_ELG;
-   case GCRY_PK_ECDSA: return GCRY_PK_ECC;
-   case GCRY_PK_EDDSA: return GCRY_PK_ECC;
-   case GCRY_PK_ECDH:  return GCRY_PK_ECC;
-   default:            return algo;
+   case GCRY_PK_ECDSA:
+   case GCRY_PK_ECDH:
+     return GCRY_PK_ECC;
+   case GCRY_PK_ELG_E:
+     return GCRY_PK_ELG;
+   default:
+     return algo;
    }
 }
 
 
+
 /* Return the spec structure for the public key algorithm ALGO.  For
    an unknown algorithm NULL is returned.  */
 static gcry_pk_spec_t *
@@ -116,7 +117,7 @@ spec_from_name (const char *name)
  * set the function will only succeed if a private key has been given.
  * On success the spec is stored at R_SPEC.  On error NULL is stored
  * at R_SPEC and an error code returned.  If R_PARMS is not NULL and
- * the function returns success, the parameter list below
+ * the fucntion returns success, the parameter list below
  * "private-key" or "public-key" is stored there and the caller must
  * call gcry_sexp_release on it.
  */
@@ -196,8 +197,6 @@ _gcry_pk_map_name (const char *string)
     return 0;
   if (spec->flags.disabled)
     return 0;
-  if (!spec->flags.fips && fips_mode ())
-    return 0;
   return spec->algo;
 }
 
@@ -227,7 +226,7 @@ check_pubkey_algo (int algo, unsigned use)
   gcry_pk_spec_t *spec;
 
   spec = spec_from_algo (algo);
-  if (spec && !spec->flags.disabled && (spec->flags.fips || !fips_mode ()))
+  if (spec)
     {
       if (((use & GCRY_PK_USAGE_SIGN)
 	   && (! (spec->use & GCRY_PK_USAGE_SIGN)))
@@ -296,7 +295,7 @@ pubkey_get_nenc (int algo)
    Caller has to provide a public key as the SEXP pkey and data as a
    SEXP with just one MPI in it. Alternatively S_DATA might be a
    complex S-Expression, similar to the one used for signature
-   verification.  This provides a flag which allows handling PKCS#1
+   verification.  This provides a flag which allows to handle PKCS#1
    block type 2 padding.  The function returns a sexp which may be
    passed to to pk_decrypt.
 
@@ -325,19 +324,6 @@ _gcry_pk_encrypt (gcry_sexp_t *r_ciph, gcry_sexp_t s_data, gcry_sexp_t s_pkey)
   if (rc)
     goto leave;
 
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
   if (spec->encrypt)
     rc = spec->encrypt (r_ciph, s_data, keyparms);
   else
@@ -390,11 +376,7 @@ _gcry_pk_decrypt (gcry_sexp_t *r_plain, gcry_sexp_t s_data, gcry_sexp_t s_skey)
   if (rc)
     goto leave;
 
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (spec->decrypt)
+  if (spec->decrypt)
     rc = spec->decrypt (r_plain, s_data, keyparms);
   else
     rc = GPG_ERR_NOT_IMPLEMENTED;
@@ -447,19 +429,6 @@ _gcry_pk_sign (gcry_sexp_t *r_sig, gcry_sexp_t s_hash, gcry_sexp_t s_skey)
   if (rc)
     goto leave;
 
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
   if (spec->sign)
     rc = spec->sign (r_sig, s_hash, keyparms);
   else
@@ -471,212 +440,6 @@ _gcry_pk_sign (gcry_sexp_t *r_sig, gcry_sexp_t s_hash, gcry_sexp_t s_skey)
 }
 
 
-#define MAX_CONTEXTS 2 /* Currently, input data and random_override */
-
-static gcry_err_code_t
-prepare_datasexp_to_be_signed (const char *tmpl, gcry_md_hd_t hd,
-                               gcry_ctx_t ctx, gcry_sexp_t *s_data_p)
-{
-  const char *s;
-  const char *digest_name = NULL;
-  const unsigned char *digest;
-  int digest_size;
-  int algo;
-  gcry_err_code_t rc;
-
-  if (hd == NULL)
-    {
-      const unsigned char *data[MAX_CONTEXTS];
-      int data_size[MAX_CONTEXTS];
-      int i = 0;
-      void *argv[MAX_CONTEXTS*2];
-
-      while (1)
-        {
-          size_t len;
-
-          rc = _gcry_pk_get_single_data (&ctx, &data[i/2], &len);
-          if (rc)
-            return rc;
-
-          data_size[i/2] = (int)len;
-
-          argv[i] = (void *)&data_size[i/2];
-          argv[i+1] = (void *)&data[i/2];
-
-          i += 2;
-
-          if (!ctx)
-            break;
-
-          if (i >= MAX_CONTEXTS*2)
-            return GPG_ERR_EINVAL;
-        }
-
-      rc = _gcry_sexp_build_array (s_data_p, NULL, tmpl, argv);
-      return rc;
-    }
-
-  /* Check if it has fixed hash name or %s */
-  s = strstr (tmpl, "(hash ");
-  if (s == NULL)
-    {
-      _gcry_md_close (hd);
-      return GPG_ERR_DIGEST_ALGO;
-    }
-
-  s += 6;
-  if (!strncmp (s, "%s", 2))
-    {
-      algo = _gcry_md_get_algo (hd);
-
-      if (fips_mode () && algo == GCRY_MD_SHA1)
-        {
-          if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-            {
-              _gcry_md_close (hd);
-              return GPG_ERR_DIGEST_ALGO;
-            }
-          else
-            fips_service_indicator_mark_non_compliant ();
-        }
-
-      digest_name = _gcry_md_algo_name (algo);
-      digest_size = (int)_gcry_md_get_algo_dlen (algo);
-      digest = _gcry_md_read (hd, 0);
-    }
-  else
-    {
-      const char *p;
-      char *digest_name_supplied;
-
-      for (p = s; *p && *p != ' '; p++)
-	;
-
-      digest_name_supplied = xtrymalloc (p - s + 1);
-      if (!digest_name_supplied)
-        {
-          rc = gpg_err_code_from_syserror ();
-          _gcry_md_close (hd);
-          return rc;
-        }
-      memcpy (digest_name_supplied, s, p - s);
-      digest_name_supplied[p - s] = 0;
-
-      algo = _gcry_md_map_name (digest_name_supplied);
-      xfree (digest_name_supplied);
-      if (algo == 0)
-	{
-	  _gcry_md_close (hd);
-	  return GPG_ERR_DIGEST_ALGO;
-	}
-      else if (fips_mode () && algo == GCRY_MD_SHA1)
-        {
-          if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-            {
-              _gcry_md_close (hd);
-              return GPG_ERR_DIGEST_ALGO;
-            }
-          else
-            fips_service_indicator_mark_non_compliant ();
-        }
-
-      digest_size = (int)_gcry_md_get_algo_dlen (algo);
-      digest = _gcry_md_read (hd, algo);
-    }
-
-  if (!digest)
-    {
-      _gcry_md_close (hd);
-      return GPG_ERR_NOT_IMPLEMENTED;
-    }
-
-  if (!ctx)
-    {
-      if (!digest_name)
-	rc = _gcry_sexp_build (s_data_p, NULL, tmpl,
-			       digest_size, digest);
-      else
-	rc = _gcry_sexp_build (s_data_p, NULL, tmpl, digest_name,
-			       digest_size, digest);
-    }
-  else
-    {
-      const unsigned char *p;
-      size_t len;
-
-      rc = _gcry_pk_get_single_data (&ctx, &p, &len);
-      if (rc)
-        return rc;
-
-      if (!digest_name)
-	rc = _gcry_sexp_build (s_data_p, NULL, tmpl,
-                               digest_size, digest, (int) len, p);
-      else
-	rc = _gcry_sexp_build (s_data_p, NULL, tmpl, digest_name,
-                               digest_size, digest, (int) len, p);
-    }
-
-  _gcry_md_close (hd);
-  return rc;
-}
-
-
-gcry_err_code_t
-_gcry_pk_sign_md (gcry_sexp_t *r_sig, const char *tmpl, gcry_md_hd_t hd_orig,
-                  gcry_sexp_t s_skey, gcry_ctx_t ctx)
-{
-  gcry_err_code_t rc;
-  gcry_pk_spec_t *spec;
-  gcry_sexp_t keyparms = NULL;
-  gcry_sexp_t s_data = NULL;
-  gcry_error_t err;
-  gcry_md_hd_t hd;
-
-  *r_sig = NULL;
-
-  if (!hd_orig)
-    hd = NULL;
-  else
-    {
-      err = _gcry_md_copy (&hd, hd_orig);
-      if (err)
-        return gpg_err_code (err);
-    }
-
-  rc = prepare_datasexp_to_be_signed (tmpl, hd, ctx, &s_data);
-  if (rc)
-    return rc;
-
-  rc = spec_from_sexp (s_skey, 1, &spec, &keyparms);
-  if (rc)
-    goto leave;
-
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
-  if (spec->sign)
-    rc = spec->sign (r_sig, s_data, keyparms);
-  else
-    rc = GPG_ERR_NOT_IMPLEMENTED;
-
- leave:
-  sexp_release (s_data);
-  sexp_release (keyparms);
-  return rc;
-}
-
-
 /*
    Verify a signature.
 
@@ -695,19 +458,6 @@ _gcry_pk_verify (gcry_sexp_t s_sig, gcry_sexp_t s_hash, gcry_sexp_t s_pkey)
   if (rc)
     goto leave;
 
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
   if (spec->verify)
     rc = spec->verify (s_sig, s_hash, keyparms);
   else
@@ -719,59 +469,6 @@ _gcry_pk_verify (gcry_sexp_t s_sig, gcry_sexp_t s_hash, gcry_sexp_t s_pkey)
 }
 
 
-gcry_err_code_t
-_gcry_pk_verify_md (gcry_sexp_t s_sig, const char *tmpl, gcry_md_hd_t hd_orig,
-                    gcry_sexp_t s_pkey, gcry_ctx_t ctx)
-{
-  gcry_err_code_t rc;
-  gcry_pk_spec_t *spec;
-  gcry_sexp_t keyparms = NULL;
-  gcry_sexp_t s_data = NULL;
-  gcry_error_t err;
-  gcry_md_hd_t hd;
-
-  if (!hd_orig)
-    hd = NULL;
-  else
-    {
-      err = _gcry_md_copy (&hd, hd_orig);
-      if (err)
-        return gpg_err_code (err);
-    }
-
-  rc = prepare_datasexp_to_be_signed (tmpl, hd, ctx, &s_data);
-  if (rc)
-    return rc;
-
-  rc = spec_from_sexp (s_pkey, 0, &spec, &keyparms);
-  if (rc)
-    goto leave;
-
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
-  if (spec->verify)
-    rc = spec->verify (s_sig, s_data, keyparms);
-  else
-    rc = GPG_ERR_NOT_IMPLEMENTED;
-
- leave:
-  sexp_release (s_data);
-  sexp_release (keyparms);
-  return rc;
-}
-
-
 /*
    Test a key.
 
@@ -792,19 +489,6 @@ _gcry_pk_testkey (gcry_sexp_t s_key)
   if (rc)
     goto leave;
 
-  if (spec->flags.disabled)
-    rc = GPG_ERR_PUBKEY_ALGO;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
-
   if (spec->check_secret_key)
     rc = spec->check_secret_key (keyparms);
   else
@@ -816,34 +500,38 @@ _gcry_pk_testkey (gcry_sexp_t s_key)
 }
 
 
-
-/* Create a public key pair and return it as R_KEY.
- * How the key is created depends on s_parms:
- *
- *   (genkey
- *     (algo
- *       (parameter_name_1 ....)
- *       ....
- *       (parameter_name_n ....)))
- *
- * The key is returned in a format depending on the algorithm. Both,
- * private and secret keys are returned and optionally some additional
- * information.  For example for Elgamal this structure is returned:
- *
- *   (key-data
- *     (public-key
- *       (elg
- *         (p <mpi>)
- *         (g <mpi>)
- *         (y <mpi>)))
- *     (private-key
- *       (elg
- *         (p <mpi>)
- *         (g <mpi>)
- *         (y <mpi>)
- *         (x <mpi>)))
- *     (misc-key-info
- *        (pm1-factors n1 n2 ... nn)))
+/*
+  Create a public key pair and return it in r_key.
+  How the key is created depends on s_parms:
+  (genkey
+   (algo
+     (parameter_name_1 ....)
+      ....
+     (parameter_name_n ....)
+  ))
+  The key is returned in a format depending on the
+  algorithm. Both, private and secret keys are returned
+  and optionally some additional informatin.
+  For elgamal we return this structure:
+  (key-data
+   (public-key
+     (elg
+ 	(p <mpi>)
+ 	(g <mpi>)
+ 	(y <mpi>)
+     )
+   )
+   (private-key
+     (elg
+ 	(p <mpi>)
+ 	(g <mpi>)
+ 	(y <mpi>)
+ 	(x <mpi>)
+     )
+   )
+   (misc-key-info
+      (pm1-factors n1 n2 ... nn)
+   ))
  */
 gcry_err_code_t
 _gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
@@ -883,21 +571,11 @@ _gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
   spec = spec_from_name (name);
   xfree (name);
   name = NULL;
-  if (!spec || spec->flags.disabled)
+  if (!spec)
     {
       rc = GPG_ERR_PUBKEY_ALGO; /* Unknown algorithm.  */
       goto leave;
     }
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        {
-          rc = GPG_ERR_PUBKEY_ALGO;
-          goto leave;
-        }
-      else
-        fips_service_indicator_mark_non_compliant ();
-    }
 
   if (spec->generate)
     rc = spec->generate (list, r_key);
@@ -934,21 +612,7 @@ _gcry_pk_get_nbits (gcry_sexp_t key)
   if (spec_from_sexp (key, 0, &spec, &parms))
     return 0; /* Error - 0 is a suitable indication for that.  */
 
-  if (spec->flags.disabled)
-    nbits = 0;                  /* Error */
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        nbits = 0;              /* Error */
-      else
-        {
-          fips_service_indicator_mark_non_compliant ();
-          nbits = spec->get_nbits (parms);
-        }
-    }
-  else
-    nbits = spec->get_nbits (parms);
-
+  nbits = spec->get_nbits (parms);
   sexp_release (parms);
   return nbits;
 }
@@ -1080,19 +744,7 @@ _gcry_pk_get_curve (gcry_sexp_t key, int iterator, unsigned int *r_nbits)
         return NULL;
     }
 
-  if (spec->flags.disabled)
-    result = NULL;
-  else if (!spec->flags.fips && fips_mode ())
-    {
-      if (fips_check_rejection (GCRY_FIPS_FLAG_REJECT_PK))
-        result = NULL;
-      else
-        {
-          fips_service_indicator_mark_non_compliant ();
-          result = spec->get_curve (keyparms, iterator, r_nbits);
-        }
-    }
-  else if (spec->get_curve)
+  if (spec->get_curve)
     result = spec->get_curve (keyparms, iterator, r_nbits);
 
   sexp_release (keyparms);
@@ -1291,19 +943,16 @@ _gcry_pk_selftest (int algo, int extended, selftest_report_func_t report)
 
   algo = map_algo (algo);
   spec = spec_from_algo (algo);
-  if (spec && !spec->flags.disabled
-      && (spec->flags.fips || !fips_mode ())
-      && spec->selftest)
+  if (spec && !spec->flags.disabled && spec->selftest)
     ec = spec->selftest (algo, extended, report);
   else
     {
       ec = GPG_ERR_PUBKEY_ALGO;
-      /* Fixme: We need to change the report function to allow passing
+      /* Fixme: We need to change the report fucntion to allow passing
          of an encryption mode (e.g. pkcs1, ecdsa, or ecdh).  */
       if (report)
         report ("pubkey", algo, "module",
-                spec && !spec->flags.disabled
-                && (spec->flags.fips || !fips_mode ())?
+                spec && !spec->flags.disabled?
                 "no selftest available" :
                 spec? "algorithm disabled" :
                 "algorithm not found");
@@ -1311,52 +960,3 @@ _gcry_pk_selftest (int algo, int extended, selftest_report_func_t report)
 
   return gpg_error (ec);
 }
-
-
-struct pk_single_data {
-  size_t len;
-  unsigned char area[1];  /* In future, we may use flexible array member.  */
-};
-
-gpg_err_code_t
-_gcry_pk_single_data_push (gcry_ctx_t *r_ctx,
-                          const unsigned char *p, size_t len)
-{
-  gcry_ctx_t ctx;
-  struct pk_single_data *psd;
-  int data_type = CONTEXT_TYPE_SINGLE_DATA;
-
-  if (!p)
-    return GPG_ERR_EINVAL;
-
-  ctx = _gcry_ctx_alloc (data_type,
-			 offsetof (struct pk_single_data, area) + len,
-                         NULL, *r_ctx);
-  if (!ctx)
-    return gpg_err_code_from_syserror ();
-  psd = _gcry_ctx_get_pointer (ctx, data_type);
-  psd->len = len;
-  memcpy (psd->area, p, len);
-
-  *r_ctx = ctx;
-  return 0;
-}
-
-gpg_err_code_t
-_gcry_pk_get_single_data (gcry_ctx_t *r_ctx,
-                          const unsigned char **r_p, size_t *r_len)
-{
-  struct pk_single_data *psd;
-  int data_type = CONTEXT_TYPE_SINGLE_DATA;
-  gcry_ctx_t ctx = *r_ctx;
-
-  psd = _gcry_ctx_find_pointer (ctx, data_type);
-  if (!psd)
-    return GPG_ERR_EINVAL;
-
-  *r_p = psd->area;
-  *r_len = psd->len;
-  *r_ctx = _gcry_ctx_get_pointer (ctx, 0);
-
-  return 0;
-}
