@@ -40,58 +40,16 @@
 #include "bithelp.h"
 #include "bufhelp.h"
 #include "cipher.h"
-#include "sha1.h"
+#include "hash-common.h"
 
 
 /* USE_SSSE3 indicates whether to compile with Intel SSSE3 code. */
 #undef USE_SSSE3
-#if defined(__x86_64__) && defined(HAVE_GCC_INLINE_ASM_SSSE3) && \
-    (defined(HAVE_COMPATIBLE_GCC_AMD64_PLATFORM_AS) || \
-     defined(HAVE_COMPATIBLE_GCC_WIN64_PLATFORM_AS))
+#if defined(__x86_64__) && defined(HAVE_COMPATIBLE_GCC_AMD64_PLATFORM_AS) && \
+    defined(HAVE_GCC_INLINE_ASM_SSSE3)
 # define USE_SSSE3 1
 #endif
 
-/* USE_AVX indicates whether to compile with Intel AVX code. */
-#undef USE_AVX
-#if defined(__x86_64__) && defined(HAVE_GCC_INLINE_ASM_AVX) && \
-    (defined(HAVE_COMPATIBLE_GCC_AMD64_PLATFORM_AS) || \
-     defined(HAVE_COMPATIBLE_GCC_WIN64_PLATFORM_AS))
-# define USE_AVX 1
-#endif
-
-/* USE_BMI2 indicates whether to compile with Intel AVX/BMI2 code. */
-#undef USE_BMI2
-#if defined(__x86_64__) && defined(HAVE_GCC_INLINE_ASM_AVX) && \
-    defined(HAVE_GCC_INLINE_ASM_BMI2) && \
-    (defined(HAVE_COMPATIBLE_GCC_AMD64_PLATFORM_AS) || \
-     defined(HAVE_COMPATIBLE_GCC_WIN64_PLATFORM_AS))
-# define USE_BMI2 1
-#endif
-
-/* USE_NEON indicates whether to enable ARM NEON assembly code. */
-#undef USE_NEON
-#ifdef ENABLE_NEON_SUPPORT
-# if defined(HAVE_ARM_ARCH_V6) && defined(__ARMEL__) \
-     && defined(HAVE_COMPATIBLE_GCC_ARM_PLATFORM_AS) \
-     && defined(HAVE_GCC_INLINE_ASM_NEON)
-#  define USE_NEON 1
-# endif
-#endif
-
-/* USE_ARM_CE indicates whether to enable ARMv8 Crypto Extension assembly
- * code. */
-#undef USE_ARM_CE
-#ifdef ENABLE_ARM_CRYPTO_SUPPORT
-# if defined(HAVE_ARM_ARCH_V6) && defined(__ARMEL__) \
-     && defined(HAVE_COMPATIBLE_GCC_ARM_PLATFORM_AS) \
-     && defined(HAVE_GCC_INLINE_ASM_AARCH32_CRYPTO)
-#  define USE_ARM_CE 1
-# elif defined(__AARCH64EL__) \
-       && defined(HAVE_COMPATIBLE_GCC_AARCH64_PLATFORM_AS) \
-       && defined(HAVE_GCC_INLINE_ASM_AARCH64_CRYPTO)
-#  define USE_ARM_CE 1
-# endif
-#endif
 
 /* A macro to test whether P is properly aligned for an u32 type.
    Note that config.h provides a suitable replacement for uintptr_t if
@@ -102,16 +60,23 @@
 /* # define U32_ALIGNED_P(p) (!(((uintptr_t)p) % sizeof (u32))) */
 /* #endif */
 
+typedef struct
+{
+  gcry_md_block_ctx_t bctx;
+  u32           h0,h1,h2,h3,h4;
+#ifdef USE_SSSE3
+  unsigned int use_ssse3:1;
+#endif
+} SHA1_CONTEXT;
 
 static unsigned int
-transform (void *c, const unsigned char *data, size_t nblks);
+transform (void *c, const unsigned char *data);
 
 
 static void
 sha1_init (void *context, unsigned int flags)
 {
   SHA1_CONTEXT *hd = context;
-  unsigned int features = _gcry_get_hw_features ();
 
   (void)flags;
 
@@ -128,34 +93,8 @@ sha1_init (void *context, unsigned int flags)
   hd->bctx.bwrite = transform;
 
 #ifdef USE_SSSE3
-  hd->use_ssse3 = (features & HWF_INTEL_SSSE3) != 0;
+  hd->use_ssse3 = (_gcry_get_hw_features () & HWF_INTEL_SSSE3) != 0;
 #endif
-#ifdef USE_AVX
-  /* AVX implementation uses SHLD which is known to be slow on non-Intel CPUs.
-   * Therefore use this implementation on Intel CPUs only. */
-  hd->use_avx = (features & HWF_INTEL_AVX) && (features & HWF_INTEL_FAST_SHLD);
-#endif
-#ifdef USE_BMI2
-  hd->use_bmi2 = (features & HWF_INTEL_AVX) && (features & HWF_INTEL_BMI2);
-#endif
-#ifdef USE_NEON
-  hd->use_neon = (features & HWF_ARM_NEON) != 0;
-#endif
-#ifdef USE_ARM_CE
-  hd->use_arm_ce = (features & HWF_ARM_SHA1) != 0;
-#endif
-  (void)features;
-}
-
-/*
- * Initialize the context HD. This is used to prepare the use of
- * _gcry_sha1_mixblock.  WARNING: This is a special purpose function
- * for exclusive use by random-csprng.c.
- */
-void
-_gcry_sha1_mixblock_init (SHA1_CONTEXT *hd)
-{
-  sha1_init (hd, 0);
 }
 
 
@@ -181,23 +120,11 @@ _gcry_sha1_mixblock_init (SHA1_CONTEXT *hd)
 			       } while(0)
 
 
-#ifdef USE_NEON
-unsigned int
-_gcry_sha1_transform_armv7_neon (void *state, const unsigned char *data,
-                                 size_t nblks);
-#endif
-
-#ifdef USE_ARM_CE
-unsigned int
-_gcry_sha1_transform_armv8_ce (void *state, const unsigned char *data,
-                               size_t nblks);
-#endif
-
 /*
  * Transform NBLOCKS of each 64 bytes (16 32-bit words) at DATA.
  */
 static unsigned int
-transform_blk (void *ctx, const unsigned char *data)
+_transform (void *ctx, const unsigned char *data)
 {
   SHA1_CONTEXT *hd = ctx;
   const u32 *idata = (const void *)data;
@@ -307,114 +234,24 @@ transform_blk (void *ctx, const unsigned char *data)
 }
 
 
-/* Assembly implementations use SystemV ABI, ABI conversion and additional
- * stack to store XMM6-XMM15 needed on Win64. */
-#undef ASM_FUNC_ABI
-#undef ASM_EXTRA_STACK
-#if defined(USE_SSSE3) || defined(USE_AVX) || defined(USE_BMI2)
-# ifdef HAVE_COMPATIBLE_GCC_WIN64_PLATFORM_AS
-#  define ASM_FUNC_ABI __attribute__((sysv_abi))
-#  define ASM_EXTRA_STACK (10 * 16)
-# else
-#  define ASM_FUNC_ABI
-#  define ASM_EXTRA_STACK 0
-# endif
-#endif
-
-
 #ifdef USE_SSSE3
 unsigned int
-_gcry_sha1_transform_amd64_ssse3 (void *state, const unsigned char *data,
-                                  size_t nblks) ASM_FUNC_ABI;
-#endif
-
-#ifdef USE_AVX
-unsigned int
-_gcry_sha1_transform_amd64_avx (void *state, const unsigned char *data,
-                                 size_t nblks) ASM_FUNC_ABI;
-#endif
-
-#ifdef USE_BMI2
-unsigned int
-_gcry_sha1_transform_amd64_avx_bmi2 (void *state, const unsigned char *data,
-                                     size_t nblks) ASM_FUNC_ABI;
+_gcry_sha1_transform_amd64_ssse3 (void *state, const unsigned char *data);
 #endif
 
 
 static unsigned int
-transform (void *ctx, const unsigned char *data, size_t nblks)
+transform (void *ctx, const unsigned char *data)
 {
   SHA1_CONTEXT *hd = ctx;
-  unsigned int burn;
 
-#ifdef USE_BMI2
-  if (hd->use_bmi2)
-    return _gcry_sha1_transform_amd64_avx_bmi2 (&hd->h0, data, nblks)
-           + 4 * sizeof(void*) + ASM_EXTRA_STACK;
-#endif
-#ifdef USE_AVX
-  if (hd->use_avx)
-    return _gcry_sha1_transform_amd64_avx (&hd->h0, data, nblks)
-           + 4 * sizeof(void*) + ASM_EXTRA_STACK;
-#endif
 #ifdef USE_SSSE3
   if (hd->use_ssse3)
-    return _gcry_sha1_transform_amd64_ssse3 (&hd->h0, data, nblks)
-           + 4 * sizeof(void*) + ASM_EXTRA_STACK;
-#endif
-#ifdef USE_ARM_CE
-  if (hd->use_arm_ce)
-    return _gcry_sha1_transform_armv8_ce (&hd->h0, data, nblks);
-#endif
-#ifdef USE_NEON
-  if (hd->use_neon)
-    return _gcry_sha1_transform_armv7_neon (&hd->h0, data, nblks)
+    return _gcry_sha1_transform_amd64_ssse3 (&hd->h0, data)
            + 4 * sizeof(void*);
 #endif
 
-  do
-    {
-      burn = transform_blk (hd, data);
-      data += 64;
-    }
-  while (--nblks);
-
-#ifdef ASM_EXTRA_STACK
-  /* 'transform_blk' is typically inlined and XMM6-XMM15 are stored at
-   *  the prologue of this function. Therefore need to add ASM_EXTRA_STACK to
-   *  here too.
-   */
-  burn += ASM_EXTRA_STACK;
-#endif
-
-  return burn;
-}
-
-
-/*
- * Apply the SHA-1 transform function on the buffer BLOCKOF64BYTE
- * which must have a length 64 bytes.  BLOCKOF64BYTE must be 32-bit
- * aligned.  Updates the 20 bytes in BLOCKOF64BYTE with its mixed
- * content.  Returns the number of bytes which should be burned on the
- * stack.  You need to use _gcry_sha1_mixblock_init to initialize the
- * context.
- * WARNING: This is a special purpose function for exclusive use by
- * random-csprng.c.
- */
-unsigned int
-_gcry_sha1_mixblock (SHA1_CONTEXT *hd, void *blockof64byte)
-{
-  u32 *p = blockof64byte;
-  unsigned int nburn;
-
-  nburn = transform (hd, blockof64byte, 1);
-  p[0] = hd->h0;
-  p[1] = hd->h1;
-  p[2] = hd->h2;
-  p[3] = hd->h3;
-  p[4] = hd->h4;
-
-  return nburn;
+  return _transform (hd, data);
 }
 
 
@@ -471,11 +308,11 @@ sha1_final(void *context)
   /* append the 64 bit count */
   buf_put_be32(hd->bctx.buf + 56, msb);
   buf_put_be32(hd->bctx.buf + 60, lsb);
-  burn = transform( hd, hd->bctx.buf, 1 );
+  burn = transform( hd, hd->bctx.buf );
   _gcry_burn_stack (burn);
 
   p = hd->bctx.buf;
-#define X(a) do { buf_put_be32(p, hd->h##a); p += 4; } while(0)
+#define X(a) do { *(u32*)p = be_bswap32(hd->h##a) ; p += 4; } while(0)
   X(0);
   X(1);
   X(2);
@@ -620,7 +457,7 @@ gcry_md_spec_t _gcry_digest_spec_sha1 =
   {
     GCRY_MD_SHA1, {0, 1},
     "SHA1", asn, DIM (asn), oid_spec_sha1, 20,
-    sha1_init, _gcry_md_block_write, sha1_final, sha1_read, NULL,
+    sha1_init, _gcry_md_block_write, sha1_final, sha1_read,
     sizeof (SHA1_CONTEXT),
     run_selftests
   };
