@@ -4,7 +4,7 @@
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
+ * it under the terms of the GNU Lesser general Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
@@ -24,6 +24,9 @@
 #define G10_CIPHER_PROTO_H
 
 
+enum pk_encoding;
+
+
 /* Definition of a function used to report selftest failures.
    DOMAIN is a string describing the function block:
           "cipher", "digest", "pubkey or "random",
@@ -89,7 +92,7 @@ typedef const char *(*pk_get_curve_t)(gcry_sexp_t keyparms, int iterator,
 typedef gcry_sexp_t (*pk_get_curve_param_t)(const char *name);
 
 
-/* Module specification structure for public key algorithms.  */
+/* Module specification structure for public key algoritms.  */
 typedef struct gcry_pk_spec
 {
   int algo;
@@ -126,13 +129,10 @@ typedef struct gcry_pk_spec
  *
  */
 
-struct cipher_bulk_ops;
-
 /* Type for the cipher_setkey function.  */
 typedef gcry_err_code_t (*gcry_cipher_setkey_t) (void *c,
 						 const unsigned char *key,
-						 unsigned keylen,
-						 struct cipher_bulk_ops *bulk_ops);
+						 unsigned keylen);
 
 /* Type for the cipher_encrypt function.  */
 typedef unsigned int (*gcry_cipher_encrypt_t) (void *c,
@@ -181,7 +181,7 @@ typedef struct gcry_cipher_spec
   } flags;
   const char *name;
   const char **aliases;
-  const gcry_cipher_oid_spec_t *oids;
+  gcry_cipher_oid_spec_t *oids;
   size_t blocksize;
   size_t keylen;
   size_t contextsize;
@@ -215,15 +215,6 @@ typedef void (*gcry_md_final_t) (void *c);
 /* Type for the md_read function.  */
 typedef unsigned char *(*gcry_md_read_t) (void *c);
 
-/* Type for the md_extract function.  */
-typedef gpg_err_code_t (*gcry_md_extract_t) (void *c, void *outbuf,
-					     size_t nbytes);
-
-/* Type for the md_hash_buffers function. */
-typedef void (*gcry_md_hash_buffers_t) (void *outbuf, size_t nbytes,
-					const gcry_buffer_t *iov,
-					int iovcnt);
-
 typedef struct gcry_md_oid_spec
 {
   const char *oidstring;
@@ -238,16 +229,14 @@ typedef struct gcry_md_spec
     unsigned int fips:1;
   } flags;
   const char *name;
-  const unsigned char *asnoid;
+  unsigned char *asnoid;
   int asnlen;
-  const gcry_md_oid_spec_t *oids;
+  gcry_md_oid_spec_t *oids;
   int mdlen;
   gcry_md_init_t init;
   gcry_md_write_t write;
   gcry_md_final_t final;
   gcry_md_read_t read;
-  gcry_md_extract_t extract;
-  gcry_md_hash_buffers_t hash_buffers;
   size_t contextsize; /* allocate this amount of context */
   selftest_func_t selftest;
 } gcry_md_spec_t;
@@ -261,10 +250,8 @@ gcry_error_t _gcry_md_selftest (int algo, int extended,
                                 selftest_report_func_t report);
 gcry_error_t _gcry_pk_selftest (int algo, int extended,
                                 selftest_report_func_t report);
-gcry_error_t _gcry_mac_selftest (int algo, int extended,
-                                 selftest_report_func_t report);
-gcry_error_t _gcry_kdf_selftest (int algo, int extended,
-                                 selftest_report_func_t report);
+gcry_error_t _gcry_hmac_selftest (int algo, int extended,
+                                  selftest_report_func_t report);
 
 gcry_error_t _gcry_random_selftest (selftest_report_func_t report);
 
